<?php

namespace Drupal\smart_content_segments\Entity;
use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\smart_content\Condition\ConditionInterface;
use Drupal\smart_content\Condition\ConditionsHelperTrait;
use Drupal\smart_content\Entity\SmartVariationSet;
use Drupal\smart_content\Segment;

/**
 * Defines the Smart segment entity.
 *
 * @ConfigEntityType(
 *   id = "smart_segment",
 *   label = @Translation("Smart segment"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\smart_content_segments\SmartSegmentListBuilder",
 *     "form" = {
 *       "add" = "Drupal\smart_content_segments\Form\SmartSegmentForm",
 *       "edit" = "Drupal\smart_content_segments\Form\SmartSegmentForm",
 *       "delete" = "Drupal\smart_content_segments\Form\SmartSegmentDeleteForm"
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\smart_content_segments\SmartSegmentHtmlRouteProvider",
 *     },
 *   },
 *   config_prefix = "smart_segment",
 *   admin_permission = "administer smart segments",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid"
 *   },
 *   links = {
 *     "canonical" = "/admin/structure/smart_segment/{smart_segment}",
 *     "add-form" = "/admin/structure/smart_segment/add",
 *     "edit-form" = "/admin/structure/smart_segment/{smart_segment}/edit",
 *     "delete-form" = "/admin/structure/smart_segment/{smart_segment}/delete",
 *     "collection" = "/admin/structure/smart_segment"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "conditions",
 *   },
 * )
 */
class SmartSegment extends ConfigEntityBase implements SmartSegmentInterface {

  use ConditionsHelperTrait;

  /**
   * The Smart segment ID.
   *
   * @var string
   */
  protected $id;

  /**
   * The Smart segment label.
   *
   * @var string
   */
  protected $label;

  public function writeChangesToSettings() {
    // Ensure conditions are loaded from setting, then wipe settings.
    $this->conditions = $this->getConditionPluginCollection()->getConfiguration();
  }

  /**
   * Gets the plugin collections used by this object.
   *
   * @return \Drupal\Component\Plugin\LazyPluginCollection[]
   *   An array of plugin collections, keyed by the property name they use to
   *   store their configuration.
   */
  public function getPluginCollections() {
    return [
      'conditions' => $this->getConditionPluginCollection(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function save() {
    $this->writeChangesToSettings();
    $conditionManager = \Drupal::service('plugin.manager.smart_content.condition');
    $conditionManager->clearCachedDefinitions();
    parent::save();
  }

  /**
   * {@inheritdoc}
   */
  public static function postDelete(EntityStorageInterface $storage, array $entities) {
    parent::postDelete($storage, $entities);
    $conditionManager = \Drupal::service('plugin.manager.smart_content.condition');
    $conditionManager->clearCachedDefinitions();
  }


}
